@@ -8,9 +8,8 @@ import logging
 
 import numpy
 
-from implicit.als import AlternatingLeastSquares
 import implicit.cuda
-
+from implicit.als import AlternatingLeastSquares
 
 log = logging.getLogger("implicit")
 
@@ -85,14 +84,14 @@ class NMSLibAlternatingLeastSquares(AlternatingLeastSquares):
 
         super(NMSLibAlternatingLeastSquares, self).__init__(*args, **kwargs)
 
-    def fit(self, Ciu):
+    def fit(self, Ciu, show_progress=True):
         # nmslib can be a little chatty when first imported, disable some of
         # the logging
         logging.getLogger('nmslib').setLevel(logging.WARNING)
         import nmslib
 
         # train the model
-        super(NMSLibAlternatingLeastSquares, self).fit(Ciu)
+        super(NMSLibAlternatingLeastSquares, self).fit(Ciu, show_progress=show_progress)
 
         # create index for similar_items
         if self.approximate_similar_items:
@@ -112,7 +111,7 @@ class NMSLibAlternatingLeastSquares(AlternatingLeastSquares):
 
             self.similar_items_index.addDataPointBatch(item_factors, ids=ids)
             self.similar_items_index.createIndex(self.index_params,
-                                                 print_progress=self.show_progress)
+                                                 print_progress=show_progress)
             self.similar_items_index.setQueryTimeParams(self.query_params)
 
         # build up a separate index for the inner product (for recommend
@@ -124,7 +123,7 @@ class NMSLibAlternatingLeastSquares(AlternatingLeastSquares):
             self.recommend_index = nmslib.init(
                 method='hnsw', space='cosinesimil')
             self.recommend_index.addDataPointBatch(extra)
-            self.recommend_index.createIndex(self.index_params, print_progress=self.show_progress)
+            self.recommend_index.createIndex(self.index_params, print_progress=show_progress)
             self.recommend_index.setQueryTimeParams(self.query_params)
 
     def similar_items(self, itemid, N=10):
@@ -203,12 +202,12 @@ class AnnoyAlternatingLeastSquares(AlternatingLeastSquares):
         self.n_trees = n_trees
         self.search_k = search_k
 
-    def fit(self, Ciu):
+    def fit(self, Ciu, show_progress=True):
         # delay loading the annoy library in case its not installed here
         import annoy
 
         # train the model
-        super(AnnoyAlternatingLeastSquares, self).fit(Ciu)
+        super(AnnoyAlternatingLeastSquares, self).fit(Ciu, show_progress=show_progress)
 
         # build up an Annoy Index with all the item_factors (for calculating
         # similar items)
@@ -313,11 +312,11 @@ class FaissAlternatingLeastSquares(AlternatingLeastSquares):
         self.nprobe = nprobe
         super(FaissAlternatingLeastSquares, self).__init__(*args, use_gpu=use_gpu, **kwargs)
 
-    def fit(self, Ciu):
+    def fit(self, Ciu, show_progress=True):
         import faiss
 
         # train the model
-        super(FaissAlternatingLeastSquares, self).fit(Ciu)
+        super(FaissAlternatingLeastSquares, self).fit(Ciu, show_progress=show_progress)
 
         self.quantizer = faiss.IndexFlat(self.factors)
 
@@ -364,7 +363,7 @@ class FaissAlternatingLeastSquares(AlternatingLeastSquares):
             self.similar_items_index = index
 
     def similar_items(self, itemid, N=10):
-        if not self.approximate_similar_items:
+        if not self.approximate_similar_items or (self.use_gpu and N >= 1024):
             return super(FaissAlternatingLeastSquares, self).similar_items(itemid, N)
 
         factors = self.item_factors[itemid]
@@ -389,6 +388,14 @@ class FaissAlternatingLeastSquares(AlternatingLeastSquares):
             liked.update(filter_items)
         count = N + len(liked)
 
+        # the GPU variant of faiss doesn't support returning more than 1024 results.
+        # fall back to the exact match when this happens
+        if self.use_gpu and count >= 1024:
+            return super(FaissAlternatingLeastSquares,
+                         self).recommend(userid, user_items, N=N,
+                                         filter_items=filter_items,
+                                         recalculate_user=recalculate_user)
+
         # faiss expects multiple queries - convert query to a matrix
         # and results back to single vectors
         query = user.reshape(1, -1).astype('float32')
